@@ -1,7 +1,7 @@
 /*
  * Generic kernel crypto API user space interface library
  *
- * Copyright (C) 2014, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2014 - 2015, Stephan Mueller <smueller@chronox.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -64,7 +64,7 @@
 #define MINVERSION 10 /* API compatible, ABI may change, functional
 		       * enhancements only, consumer can be left unchanged if
 		       * enhancements are not considered */
-#define PATCHLEVEL 0  /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 1  /* API / ABI compatible, no functional changes, no
 		       * enhancements, bug fixes only */
 
 /* remove once in if_alg.h */
@@ -94,12 +94,33 @@
 #endif
 
 /* make sure that is equal to include/crypto/if_alg.h */
+#ifndef ALG_MAX_PAGES
 #define ALG_MAX_PAGES 16
+#endif
 
 /************************************************************
  * Internal logic
  ************************************************************/
 
+static int _kcapi_common_accept(struct kcapi_handle *handle)
+{
+	if (handle->opfd != -1)
+		return 0;
+
+	handle->opfd = accept(handle->tfmfd, NULL, 0);
+	if (handle->opfd == -1) {
+		int errsv = 0;
+
+		errsv = errno;
+		perror("AF_ALG: accept failed");
+		close(handle->tfmfd);
+		handle->tfmfd = -1;
+		return -errsv;
+	}
+
+	return 0;
+}
+
 static int32_t _kcapi_common_send_meta(struct kcapi_handle *handle,
 				       struct iovec *iov, uint32_t iovlen,
 				       uint32_t enc, uint32_t flags)
@@ -129,6 +150,10 @@ static int32_t _kcapi_common_send_meta(struct kcapi_handle *handle,
 		iv_msg_size +			/* IV */
 		assoc_msg_size;			/* AEAD associated data size */
 
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
+
 	memset(&msg, 0, sizeof(msg));
 
 	buffer = calloc(1, bufferlen);
@@ -185,6 +210,10 @@ static inline int32_t _kcapi_common_send_data(struct kcapi_handle *handle,
 	struct msghdr msg;
 	int32_t ret = 0;
 
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
+
 	msg.msg_name = NULL;
 	msg.msg_namelen = 0;
 	msg.msg_control = NULL;
@@ -206,16 +235,30 @@ static inline int32_t _kcapi_common_vmsplice_iov(struct kcapi_handle *handle,
 	uint32_t inlen = 0;
 	unsigned long i;
 
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
+
 	for (i = 0; i < iovlen; i++)
 		inlen += iov[i].iov_len;
 
+	/* kernel processes input data with max size of one page */
+	handle->processed_sg += ((inlen + sysconf(_SC_PAGESIZE) - 1) /
+				 sysconf(_SC_PAGESIZE));
+	if (handle->processed_sg > ALG_MAX_PAGES)
+		return _kcapi_common_send_data(handle, iov, iovlen,
+					       (flags & SPLICE_F_MORE) ?
+					        MSG_MORE : 0);
+
 	ret = vmsplice(handle->pipes[1], iov, iovlen, SPLICE_F_GIFT|flags);
 	if (0 > ret)
 		return ret;
-	if ((uint32_t)ret != inlen)
+	if ((uint32_t)ret != inlen) {
 		fprintf(stderr, "vmsplice: not all data received by kernel (data recieved: %ld -- data sent: %lu)\n",
 			(long)ret, (unsigned long)inlen);
-	ret = splice(handle->pipes[0], NULL, handle->opfd, NULL, ret, 0);
+		return -EFAULT;
+	}
+	ret = splice(handle->pipes[0], NULL, handle->opfd, NULL, ret, flags);
 	return (ret >= 0) ? ret : -errno;
 }
 
@@ -226,18 +269,30 @@ static inline int32_t _kcapi_common_vmsplice_chunk(struct kcapi_handle *handle,
 {
 	struct iovec iov;
 	uint32_t processed = 0;
+	int ret = 0;
+
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
 
 	while (inlen) {
 		int32_t ret = 0;
 
 		iov.iov_base = (void*)(uintptr_t)(in + processed);
 		iov.iov_len = inlen;
-		ret = vmsplice(handle->pipes[1], &iov, 1,
-			       SPLICE_F_GIFT|flags);
-		if (0 > ret)
-			return ret;
-		ret = splice(handle->pipes[0], NULL, handle->opfd, NULL, ret,
-			     flags);
+
+		if ((handle->processed_sg++) > ALG_MAX_PAGES) {
+			ret = _kcapi_common_send_data(handle, &iov, 1,
+						      (flags & SPLICE_F_MORE) ?
+						       MSG_MORE : 0);
+		} else {
+			ret = vmsplice(handle->pipes[1], &iov, 1,
+				       SPLICE_F_GIFT|flags);
+			if (0 > ret)
+				return ret;
+			ret = splice(handle->pipes[0], NULL, handle->opfd,
+				     NULL, ret, flags);
+		}
 		if (0 > ret)
 			return ret;
 
@@ -256,6 +311,10 @@ static inline int32_t _kcapi_common_recv_data(struct kcapi_handle *handle,
 	int32_t ret = 0;
 	int errsv = 0;
 
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
+
 	msg.msg_name = NULL;
 	msg.msg_namelen = 0;
 	msg.msg_control = NULL;
@@ -277,6 +336,17 @@ static inline int32_t _kcapi_common_recv_data(struct kcapi_handle *handle,
 	}
 #endif
 
+	/*
+	 * As the iovecs are processed and removed from the list in the kernel
+	 * we can also reset the list of processed iovecs here.
+	 *
+	 * Note, if there is an error, the kernel keeps the list unless it is
+	 * a "valid" error of EBADMSG indicating an integrity error of the
+	 * crypto operation.
+	 */
+	if (ret >= 0 || errsv == EBADMSG)
+		handle->processed_sg = 0;
+
 	return (ret >= 0) ? ret : -errsv;
 }
 
@@ -285,6 +355,10 @@ static inline int32_t _kcapi_common_read_data(struct kcapi_handle *handle,
 {
 	int32_t ret = 0;
 
+	ret = _kcapi_common_accept(handle);
+	if (ret)
+		return ret;
+
 	ret = read(handle->opfd, out, outlen);
 	return (ret >= 0) ? ret : -errno;
 }
@@ -517,6 +591,29 @@ err:
 	return;
 }
 
+static inline void _kcapi_aio_destroy(struct kcapi_handle *handle)
+{
+	if (handle->aio.skcipher_aio_disable)
+		return;
+	if (handle->aio.efd != -1)
+		close(handle->aio.efd);
+	io_destroy(handle->aio.aio_ctx);
+}
+
+static inline void _kcapi_handle_destroy(struct kcapi_handle *handle)
+{
+	if (handle->tfmfd != -1)
+		close(handle->tfmfd);
+	if (handle->opfd != -1)
+		close(handle->opfd);
+	if (handle->pipes[0] != -1)
+		close(handle->pipes[0]);
+	if (handle->pipes[1] != -1)
+		close(handle->pipes[1]);
+	_kcapi_aio_destroy(handle);
+	memset(handle, 0, sizeof(struct kcapi_handle));
+}
+
 static int _kcapi_handle_init(struct kcapi_handle *handle, const char *type,
 			      const char *ciphername, uint32_t flags)
 {
@@ -543,20 +640,12 @@ static int _kcapi_handle_init(struct kcapi_handle *handle, const char *type,
 		return -errsv;
 	}
 
-	handle->opfd = accept(handle->tfmfd, NULL, 0);
-	if (handle->opfd == -1) {
-		errsv = errno;
-		perror("AF_ALG: accept failed");
-		close(handle->tfmfd);
-		handle->tfmfd = -1;
-		return -errsv;
-	}
+	handle->opfd = -1;
 
 	ret = pipe(handle->pipes);
 	if (ret) {
 		errsv = errno;
 		close(handle->tfmfd);
-		close(handle->opfd);
 		return -errsv;
 	}
 
@@ -565,10 +654,7 @@ static int _kcapi_handle_init(struct kcapi_handle *handle, const char *type,
 		errsv = errno;
 		fprintf(stderr, "NETLINK_CRYPTO: cannot obtain cipher information for %s (is required crypto_user.c patch missing? see documentation)\n",
 		       ciphername);
-		close(handle->tfmfd);
-		close(handle->opfd);
-		close(handle->pipes[0]);
-		close(handle->pipes[1]);
+		_kcapi_handle_destroy(handle);
 	}
 
 	if (flags & KCAPI_INIT_AIO)
@@ -579,29 +665,6 @@ static int _kcapi_handle_init(struct kcapi_handle *handle, const char *type,
 	return (errsv) ? -errsv : ret;
 }
 
-static inline void _kcapi_aio_destroy(struct kcapi_handle *handle)
-{
-	if (handle->aio.skcipher_aio_disable)
-		return;
-	if (handle->aio.efd != -1)
-		close(handle->aio.efd);
-	io_destroy(handle->aio.aio_ctx);
-}
-
-static inline void _kcapi_handle_destroy(struct kcapi_handle *handle)
-{
-	if (handle->tfmfd != -1)
-		close(handle->tfmfd);
-	if (handle->opfd != -1)
-		close(handle->opfd);
-	if (handle->pipes[0] != -1)
-		close(handle->pipes[0]);
-	if (handle->pipes[1] != -1)
-		close(handle->pipes[1]);
-	_kcapi_aio_destroy(handle);
-	memset(handle, 0, sizeof(struct kcapi_handle));
-}
-
 /*********** Generic Helper functions *************************/
 
 void kcapi_memset_secure(void *s, int c, uint32_t n)
@@ -723,19 +786,17 @@ static int32_t _kcapi_cipher_crypt(struct kcapi_handle *handle,
 		return -EINVAL;
 	}
 
-	iov.iov_base = (void*)(uintptr_t)in;
 	/*
 	 * Using two syscalls with memcpy is faster than four syscalls
 	 * without memcpy below the given threshold.
 	 */
 	if ((access == KCAPI_ACCESS_HEURISTIC && inlen <= (1<<13)) ||
 	    access == KCAPI_ACCESS_SENDMSG) {
+		iov.iov_base = (void*)(uintptr_t)in;
 		iov.iov_len = inlen;
 		ret = _kcapi_common_send_meta(handle, &iov, 1, enc, 0);
 		if (0 > ret)
 			return ret;
-		iov.iov_base = (void*)(uintptr_t)out;
-		iov.iov_len = outlen;
 	} else {
 		ret = _kcapi_common_send_meta(handle, NULL, 0, enc, 0);
 		if (0 > ret)
@@ -753,12 +814,46 @@ static int32_t _kcapi_cipher_crypt(struct kcapi_handle *handle,
 			/* AIO not available for specific interface */
 			handle->aio.skcipher_aio_disable = 1;
 			_kcapi_aio_destroy(handle);
-			return _kcapi_common_recv_data(handle, &iov, 1);
+			return _kcapi_common_read_data(handle, out, outlen);
 		}
 		return ret;
 	}
 }
 
+static int32_t _kcapi_cipher_crypt_chunk(struct kcapi_handle *handle,
+					 const uint8_t *in, uint32_t inlen,
+					 uint8_t *out, uint32_t outlen,
+					int access, int enc)
+{
+	int32_t totallen = 0;
+
+	while (inlen) {
+		uint32_t process = inlen;
+		int32_t ret = 0;
+
+		/*
+		 * We do not check that sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES is
+		 * a multiple of blocksize, because we assume that this is
+		 * always the case.
+		 */
+		if (inlen > sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES)
+			process = sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES;
+
+		ret = _kcapi_cipher_crypt(handle, in, process, out, outlen,
+					  access, enc);
+		if (ret < 0)
+			return ret;
+
+		totallen += process;
+		in += process;
+		inlen -= process;
+		out += ret;
+		outlen -= ret;
+	}
+
+	return totallen;
+}
+
 int32_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 			     const uint8_t *in, uint32_t inlen,
 			     const uint8_t *iv,
@@ -775,8 +870,8 @@ int32_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 	}
 
 	handle->cipher.iv = iv;
-	return _kcapi_cipher_crypt(handle, in, inlen, out, outlen, access,
-				   ALG_OP_ENCRYPT);
+	return _kcapi_cipher_crypt_chunk(handle, in, inlen, out, outlen, access,
+					 ALG_OP_ENCRYPT);
 }
 
 int32_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
@@ -800,8 +895,8 @@ int32_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
 	}
 
 	handle->cipher.iv = iv;
-	return _kcapi_cipher_crypt(handle, in, inlen, out, outlen, access,
-				   ALG_OP_DECRYPT);
+	return _kcapi_cipher_crypt_chunk(handle, in, inlen, out, outlen, access,
+					 ALG_OP_DECRYPT);
 }
 
 int32_t kcapi_cipher_stream_init_enc(struct kcapi_handle *handle,
@@ -825,9 +920,9 @@ int32_t kcapi_cipher_stream_init_dec(struct kcapi_handle *handle,
 int32_t kcapi_cipher_stream_update(struct kcapi_handle *handle,
 				   struct iovec *iov, uint32_t iovlen)
 {
-	if (iovlen <= ALG_MAX_PAGES)
+	if (handle->processed_sg <= ALG_MAX_PAGES)
 		return _kcapi_common_vmsplice_iov(handle, iov, iovlen,
-						  MSG_MORE);
+						  SPLICE_F_MORE);
 	else
 		return _kcapi_common_send_data(handle, iov, iovlen, MSG_MORE);
 }
@@ -925,11 +1020,18 @@ int32_t kcapi_aead_encrypt(struct kcapi_handle *handle,
 	/* require properly sized output data size */
 	if (outlen < inlen) {
 		fprintf(stderr,
-			"AEAD Encryption: Ciphertext buffer (%lu) is smaller than plaintext buffer (%lu)\n",
-			(unsigned long)outlen, (unsigned long)inlen);
+			"AEAD Encryption: Ciphertext buffer (%u) is smaller than plaintext buffer (%u)\n",
+			outlen, inlen);
 		return -EINVAL;
 	}
 
+	if (inlen > sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES) {
+		fprintf(stderr,
+			"AEAD Encryption: Plaintext buffer (%u) is larger than maximum chunk size (%lu)\n",
+			inlen, sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES);
+		return -EMSGSIZE;
+	}
+
 	handle->cipher.iv = iv;
 
 	iov.iov_base = in;
@@ -970,11 +1072,18 @@ int32_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 	/* require properly sized output data size */
 	if (outlen < inlen) {
 		fprintf(stderr,
-			"AEAD Decryption: Plaintext buffer (%lu) is smaller than ciphertext buffer (%lu)\n",
-			(unsigned long)outlen, (unsigned long) inlen);
+			"AEAD Decryption: Plaintext buffer (%u) is smaller than ciphertext buffer (%u)\n",
+			outlen, inlen);
 		return -EINVAL;
 	}
 
+	if (inlen > sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES) {
+		fprintf(stderr,
+			"AEAD Decryption: Ciphertext buffer (%u) is larger than maximum chunk size (%lu)\n",
+			inlen, sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES);
+		return -EMSGSIZE;
+	}
+
 	handle->cipher.iv = iv;
 
 	iov.iov_base = in;
@@ -999,7 +1108,7 @@ int32_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 	if (outlen) {
 		return _kcapi_common_read_data(handle, out, outlen);
 	} else {
-		iov.iov_base = (void*)(uintptr_t)out;
+		iov.iov_base = out;
 		iov.iov_len = outlen;
 		return _kcapi_common_recv_data(handle, &iov, 1);
 	}
@@ -1026,13 +1135,20 @@ int32_t kcapi_aead_stream_init_dec(struct kcapi_handle *handle,
 int32_t kcapi_aead_stream_update(struct kcapi_handle *handle,
 				 struct iovec *iov, uint32_t iovlen)
 {
-	return _kcapi_common_send_data(handle, iov, iovlen, MSG_MORE);
+	if (handle->processed_sg <= ALG_MAX_PAGES)
+		return _kcapi_common_vmsplice_iov(handle, iov, iovlen,
+						  SPLICE_F_MORE);
+	else
+		return _kcapi_common_send_data(handle, iov, iovlen, MSG_MORE);
 }
 
 int32_t kcapi_aead_stream_update_last(struct kcapi_handle *handle,
 				      struct iovec *iov, uint32_t iovlen)
 {
-	return _kcapi_common_send_data(handle, iov, iovlen, 0);
+	if (handle->processed_sg <= ALG_MAX_PAGES)
+		return _kcapi_common_vmsplice_iov(handle, iov, iovlen, 0);
+	else
+		return _kcapi_common_send_data(handle, iov, iovlen, 0);
 }
 
 int32_t kcapi_aead_stream_op(struct kcapi_handle *handle,
@@ -1112,17 +1228,22 @@ int kcapi_md_setkey(struct kcapi_handle *handle,
 static inline int32_t _kcapi_md_update(struct kcapi_handle *handle,
 				       const uint8_t *buffer, uint32_t len)
 {
-	int32_t r;
+	int32_t ret = 0;
 
 	/* zero buffer length cannot be handled via splice */
 	/* TODO check that heuristic for sendmsg is appropriate */
-	if(len == 0 /* < (1<<15) */)
-		r = send(handle->opfd, buffer, len, MSG_MORE);
-	else
-		r = _kcapi_common_vmsplice_chunk(handle, buffer, len,
-						 SPLICE_F_MORE);
+	if(len == 0 /* < (1<<15) */) {
+		ret = _kcapi_common_accept(handle);
+		if (ret)
+			return ret;
 
-	if (r < 0 || (uint32_t)r < len)
+		ret = send(handle->opfd, buffer, len, MSG_MORE);
+	} else {
+		ret = _kcapi_common_vmsplice_chunk(handle, buffer, len,
+						   SPLICE_F_MORE);
+	}
+
+	if (ret < 0 || (uint32_t)ret < len)
 		return -EIO;
 	return 0;
 }
@@ -1309,9 +1430,9 @@ int32_t kcapi_akcipher_stream_update(struct kcapi_handle *handle,
 				     struct iovec *iov, uint32_t iovlen)
 {
 	/* TODO: vmsplice only works with ALG_MAX_PAGES - 1 -- no clue why */
-	if (iovlen < ALG_MAX_PAGES)
+	if (handle->processed_sg < ALG_MAX_PAGES)
 		return _kcapi_common_vmsplice_iov(handle, iov, iovlen,
-						  MSG_MORE);
+						  SPLICE_F_MORE);
 	else
 		return _kcapi_common_send_data(handle, iov, iovlen, MSG_MORE);
 }
@@ -1320,7 +1441,7 @@ int32_t kcapi_akcipher_stream_update_last(struct kcapi_handle *handle,
 				          struct iovec *iov, uint32_t iovlen)
 {
 	/* TODO: vmsplice only works with ALG_MAX_PAGES - 1 -- no clue why */
-	if (iovlen < ALG_MAX_PAGES)
+	if (handle->processed_sg < ALG_MAX_PAGES)
 		return _kcapi_common_vmsplice_iov(handle, iov, iovlen, 0);
 	else
 		return _kcapi_common_send_data(handle, iov, iovlen, 0);
