@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2014 - 2015, Stephan Mueller <smueller@chronox.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -124,7 +124,9 @@ struct kcapi_aead_data {
  * Cipher handle
  * @tfmfd: Socket descriptor for AF_ALG
  * @opfd: FD to open kernel crypto API TFM
- * @skdata: Common data for all ciphers
+ * @pipes: vmplice/splice pipe pair
+ * @processed_sg: number of scatter/gather entries sent to the kernel
+ * @ciper: Common data for all ciphers
  * @aead: AEAD cipher specific data
  * @info: properties of ciphers
  * @aio: AIO information
@@ -133,6 +135,7 @@ struct kcapi_handle {
 	int tfmfd;
 	int opfd;
 	int pipes[2];
+	uint32_t processed_sg;
 	struct kcapi_cipher_data cipher;
 	struct kcapi_aead_data aead;
 	struct kcapi_cipher_info info;
@@ -203,7 +206,7 @@ int kcapi_cipher_setkey(struct kcapi_handle *handle,
  * the plaintext is overwritten with the ciphertext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page boundary,
  * the vmsplice call may not send all data to the kernel.
  *
  * The IV buffer must be exactly kcapi_cipher_ivsize() bytes in size.
@@ -233,7 +236,7 @@ int32_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
  * the ciphertext is overwritten with the plaintext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page boundary,
  * the vmsplice call may not send all data to the kernel.
  *
  * The IV buffer must be exactly kcapi_cipher_ivsize() bytes in size.
@@ -328,6 +331,15 @@ int32_t kcapi_cipher_stream_init_dec(struct kcapi_handle *handle,
  * where one thread sends data to be processed and one thread picks up data
  * processed by the cipher operation.
  *
+ * IMPORTANT NOTE: The kernel will only process
+ * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES at one time. If your input data is
+ * larger than this threshold, you MUST segment it into chunks of at most
+ * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES and invoke the
+ * kcapi_cipher_stream_update() on that segment followed by
+ * kcapi_cipher_stream_op() before the next chunk is processed. If this
+ * rule is not obeyed, the thread invoking kcapi_cipher_stream_update()
+ * will be put to sleep until another thread invokes kcapi_cipher_stream_op().
+ *
  * Return: number of bytes sent to the kernel upon success;
  *	   < 0 in case of error with errno set
  */
@@ -473,7 +485,7 @@ void kcapi_aead_setassoclen(struct kcapi_handle *handle, uint32_t assoclen);
  * the plaintext is overwritten with the ciphertext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page boundary,
  * the vmsplice call may not send all data to the kernel.
  *
  * The IV buffer must be exactly kcapi_cipher_ivsize() bytes in size.
@@ -482,6 +494,10 @@ void kcapi_aead_setassoclen(struct kcapi_handle *handle, uint32_t assoclen);
  * kcapi_aead_getdata() to obtain the resulting ciphertext and authentication
  * tag references.
  *
+ * IMPORTANT NOTE: The kernel will only process
+ * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES at one time. Longer input data cannot
+ * be handled by the kernel.
+ *
  * Return: number of bytes encrypted upon success;
  *	   < 0 in case of error with errno set
  */
@@ -541,8 +557,8 @@ void kcapi_aead_getdata(struct kcapi_handle *handle,
  * the ciphertext is overwritten with the plaintext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
- * the vmsplice call may not send all data to the kernel.
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
  *
  * The IV buffer must be exactly kcapi_cipher_ivsize() bytes in size.
  *
@@ -551,6 +567,10 @@ void kcapi_aead_getdata(struct kcapi_handle *handle,
  * If this function returns < 0 and errno is set to EBADMSG, an authentication
  * error is detected.
  *
+ * IMPORTANT NOTE: The kernel will only process
+ * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES at one time. Longer input data cannot
+ * be handled by the kernel.
+ *
  * Return: number of bytes decrypted upon success;
  *	   < 0 in case of error with errno set
  */
@@ -665,6 +685,10 @@ int32_t kcapi_aead_stream_init_dec(struct kcapi_handle *handle,
  * where one thread sends data to be processed and one thread picks up data
  * processed by the cipher operation.
  *
+ * IMPORTANT NOTE: The kernel will only process
+ * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES at one time. Longer input data cannot
+ * be handled by the kernel.
+ *
  * Return: number of bytes sent to the kernel upon success;
  *	   < 0 in case of error with errno set
  */
@@ -1104,8 +1128,8 @@ int kcapi_akcipher_setpubkey(struct kcapi_handle *handle,
  * the plaintext is overwritten with the ciphertext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
- * the vmsplice call may not send all data to the kernel.
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
  *
  * If the output size is insufficiently large, -EINVAL is returned. The
  * output buffer must be at least as large as the modululs of the uses key.
@@ -1133,8 +1157,8 @@ int32_t kcapi_akcipher_encrypt(struct kcapi_handle *handle,
  * the ciphertext is overwritten with the plaintext.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
- * the vmsplice call may not send all data to the kernel.
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
  *
  * If the output size is insufficiently large, -EINVAL is returned. The
  * output buffer must be at least as large as the modululs of the uses key.
@@ -1162,8 +1186,8 @@ int32_t kcapi_akcipher_decrypt(struct kcapi_handle *handle,
  * operation, the message is overwritten with the signature.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
- * the vmsplice call may not send all data to the kernel.
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
  *
  * If the output size is insufficiently large, -EINVAL is returned. The
  * output buffer must be at least as large as the modululs of the uses key.
@@ -1191,8 +1215,8 @@ int32_t kcapi_akcipher_sign(struct kcapi_handle *handle,
  * operation, the message is overwritten with the signature.
  *
  * The memory should be aligned at the page boundary using
- * posix_memalign(PAGE_SIZE), If it is not aligned at the page boundary,
- * the vmsplice call may not send all data to the kernel.
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
  *
  * If the output size is insufficiently large, -EINVAL is returned. The
  * output buffer must be at least as large as the modululs of the uses key.
@@ -1349,6 +1373,27 @@ int32_t kcapi_akcipher_stream_init_vfy(struct kcapi_handle *handle,
  */
 int32_t kcapi_akcipher_stream_update(struct kcapi_handle *handle,
 				     struct iovec *iov, uint32_t iovlen);
+
+/**
+ * kcapi_akcipher_stream_update_last() - send last data for processing (stream)
+ * @handle: cipher handle - input
+ * @iov: scatter/gather list with data to be processed by the cipher operation.
+ *	 - input
+ * @iovlen: number of scatter/gather list elements. - input
+ *
+ * Using this function call, more data can be submitted to the kernel.
+ *
+ * This call is identical to the kcapi_akcipher_stream_update() call with the
+ * exception that it marks the last data buffer before the cipher operation
+ * is triggered.
+ *
+ * This call must be used if all data is delivered to the kernel and
+ * kcapi_akcipher_stream_op() will be invoked as a next step. This call
+ * notifies the kernel that no further data is to be expected.
+ *
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
+ */
 int32_t kcapi_akcipher_stream_update_last(struct kcapi_handle *handle,
 					  struct iovec *iov, uint32_t iovlen);
 
